use File::Basename; 
use File::Copy; 

use File::Slurp; 
use File::Spec; 
use IO::All; 
use POSIX 'strftime'; 
use Cwd qw(); 
use strict; 
use Fcntl ':mode'; 

########################################
########################################
# Start log
my $log_file = 'aleph_3rd_party_mft_patch.log'; 
open(my $log_fh, '>>', $log_file) 
  or die "Could not create the log '$log_file' $!"; 

my $revert_files_log_file = 'aleph_3rd_party_mft_patch_revert_files.log'; 
open(my $revert_files_log_fh, '>>', $revert_files_log_file) 
  or die "Could not create the log '$revert_files_log_file' $!"; 
#
########################################
########################################

########################################
# Current dir
my $orig_source_path = Cwd::cwd(); 
print $log_fh "----------------------------------------\n"; 
print $log_fh "$orig_source_path\n\n"; 
print "$orig_source_path \n\n"; print $log_fh "$orig_source_path \n\n"; 

print "\n"; 
print "==================================================\n"; 
print "   Handling /exlibris/aleph/.ssh/ . \n"; 
print "   put SSH key files\n"; 
print "==================================================\n"; 
# print "\n\n"; 

print $log_fh "------------------------------\n"; 
print $log_fh "2. Handling /exlibris/aleph/.ssh/ . \n"; 
print $log_fh "   put SSH key files\n"; 
print $log_fh "------------------------------\n\n\n"; 

########################################
########################################
# Examin the .ssh directory. 

my $aleph_ssh_dir = '/exlibris/aleph/.ssh';  # Target Dir. 

if ( -d $aleph_ssh_dir){ # (1) Directory exists 
  print "INFO Directory exists. \n"; 
  print $log_fh "INFO Directory exists. \n"; 

  if ( -w $aleph_ssh_dir){  # (2) Directory exists + writable 
    print "INFO Directory writable. :) \n"; 
    print $log_fh "INFO Directory writable. :) \n"; 

    if ( -x $aleph_ssh_dir){  # (3)  Directory exists + writable + executable 
      print "INFO Directory is already Executable. :) \n"; 
      print $log_fh "INFO Directory is already Executable. :) \n"; 
    } #  Close (3) 
  } # For (2) - going to elsif .  
  elsif ( !-w $aleph_ssh_dir || !-x $aleph_ssh_dir) { # (2) Exists, But not writable or no exe rights. 
    chmod(0700, $aleph_ssh_dir)  # chmod - sets permissions / rights for Owner (u): Read (4) + Write (2) + Exe (1) 
      or die "Could not chmod $aleph_ssh_dir : $!"; 
    print "INFO Made the Directory Executable. \n"; 
    print $log_fh "INFO Made the Directory Executable. \n"; 
  } #  Close (2) 
} # For (1) - going to elsif .  
elsif ( !-d $aleph_ssh_dir){ # Does Not Exist.
  print "INFO Directory DOES _NOT_ exist. \n"; 
  print "INFO Creating directory. \n"; 
  print $log_fh "INFO Directory DOES _NOT_ exist. \n"; 
  print $log_fh "INFO Creating directory. \n"; 

  mkdir $aleph_ssh_dir ; # Create the Dir. 

  chmod(0700, $aleph_ssh_dir) # chmod - sets permissions / rights for Owner (u): Read (4) + Write (2) + Exe (1) 
    or die "Could not chmod $aleph_ssh_dir : $!"; 
}  #  Close (1) 

########################################
########################################
# Handle the SSH key files. 
# The 3 SSH key files should be here in the same directory of this script. 

my $ssh_key_file_24 = "aleph_24_sp_mft"; 
my $ssh_key_file_24_pub = "aleph_24_sp_mft.pub"; 
my $ssh_key_file_3rd_prt_mft = "aleph_3rd_party_products_mft"; 

my @ssh_files_array = ("$ssh_key_file_24", "$ssh_key_file_24_pub", "$ssh_key_file_3rd_prt_mft" ); 

foreach my $source_ssh_file (@ssh_files_array) { 
  # print "The value of the iterator is now $_ and $source_ssh_file \n"; 
  my $full_source = "$orig_source_path/$source_ssh_file"; 
  my $full_dest = "$aleph_ssh_dir/$source_ssh_file"; 
  if (-e $full_dest ){ 
	  `rm -rf  $full_dest`;  print "Made rm -rf $full_dest \n\n"; 
  } 
  #### Do the copy 
  copy ($full_source, $full_dest)  or die "Copy failed: $!" ; 
  print "A_INFO Put: Copied SSH key file '$_'  \n"; 
  print "A_INFO  Copied  $full_source to $full_dest \n\n"; 
  chmod(0400, $full_dest) or die "Could not chmod $full_dest : $!"; 
} 

########################################
########################################
# Print files and permissions. For information. 

print "\n\nSSH key files: \n"; 
print $log_fh "\n\nSSH key files: \n"; 

while (my $f = </exlibris/aleph/.ssh/*>){ 
    # See: https://perldoc.perl.org/functions/stat.html
    # or (Perl 5.8.9): https://perldoc.perl.org/5.8.9/functions/stat.html
 if ( $f =~ /\Q$ssh_key_file_24/ ||  ## $ssh_key_file_24", "$ssh_key_file_24_pub", "$ssh_key_file_3rd_prt_mft"
      $f =~ /\Q$ssh_key_file_24_pub/ || 
      $f =~ /\Q$ssh_key_file_3rd_prt_mft/ 
       ){ 
    print "\n"; 
    my $mode2 = (stat($f))[2]; 
    printf "%04o", S_IMODE($mode2), " "; 
    print " $f\n"; 

    my $the_file_mode2 = sprintf "%04o", S_IMODE($mode2), " "; 
    print "  mode $the_file_mode2\n"; 

    printf $log_fh "%04o", S_IMODE($mode2), " "; 
    print $log_fh " $f\n"; 
  }
}

#################################################
#################################################
chdir $orig_source_path ; 
 
# print "$orig_source_path\n"; 

my $curr_dir = Cwd::cwd(); 
 
print "\n\n$curr_dir\n\n"; 

########################################
### Close log
close $log_fh; 
close $revert_files_log_fh; 

### End close log
########################################

